<?php

namespace FM\Util;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\Events\PreDispatch\Listener;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\Session\Session;
use Teknisa\Libs\Util\OAuthCustom;

class RequestListener extends Listener
{

    const OAUTH_HEADER = "OAuth-Token";
    const OAUTH_HASH = "OAuth-Hash";
    const OAUTH_PROJECT = "OAuth-Project";
    const OAUTH_SECRET_ID = "OAuth-SecretId";
    const OAUTH_KEEP_CONNECTED = "OAuth-KeepConnected";
    const KEEP_CONNECTED_POSITIVE = "Yes";

    private $httpKernel;
    private $oauth;
    private $sessionManager;
    protected $clientId;
    protected $clientSecret;

    public function __construct(Kernel $kernel, OAuthCustom $oauth, Session $sessionManager)
    {
        $this->httpKernel                   = $kernel;
        $this->oauth                        = $oauth;
        $this->sessionManager               = $sessionManager;
        $this->clientId                     = "cd7e85178153by60953518945f9hakt19a50a4";
        $this->clientSecret                 = "lib5449996c21f780fe30badd6521931a5sac9";
    }

    private function readPublicRoutes()
    {
        $urls = file_get_contents(__DIR__ . "/../../../../config/publicRoutes.json");
        $publicRoutes = json_decode($urls, true);
        return $publicRoutes;
    }

    public function preDispatch(Request $request)
    {
        try {
            $token   = $this->httpKernel->getHttpRequest()->getHeaders()->get(self::OAUTH_HEADER);

            if (!empty($token)) {
                $this->checkAccessAndSetSession($token);
            }

            $this->checkPublicRoute($request);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }
    }

    /**
     * @param $token
     */
    private function checkAccessAndSetSession($token)
    {
        $access   = $this->oauth->checkAccess($token, $this->clientSecret, array(), false);

        if (!empty($access)) {
            if ($this->sessionManager->getId() != $access['sessionId']) {
                $this->sessionManager->start();
                $this->sessionManager->destroy();
                $this->sessionManager->setId($access['sessionId']);
            }
            $this->sessionManager->start();
        }
    }

    /**
     * Returns true if the route is matched, false otherwise.
     *
     * @param string $route         The route to match.
     * @param array  $publicRoutes  The public routes array.
     */
    private function routeMatch(string $route, array $publicRoutes)
    {
        foreach ($publicRoutes as $pattern) {
            $subPatterns = [];
            $matched = preg_match_all('/#[^#]+#/', $pattern, $subPatterns);
            if ($matched) {
                $pattern = preg_replace('/#[^#]+#/', '@', $pattern);
                $pattern = str_replace('/', '\/', $pattern);
                foreach ($subPatterns[0] as $subPattern) {
                    $pattern = preg_replace('/\@/', substr($subPattern, 1, -1), $pattern, 1);
                }
                if (preg_match('/' . $pattern . '/', $route)) {
                    return true;
                }
            } else {
                if ($route === $pattern) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * @param Request $request
     * @throws \Exception
     */
    private function checkPublicRoute(Request $request)
    {
        $doesMatch = $this->routeMatch($request->getRoutePath(), $this->readPublicRoutes());
        if (!$doesMatch && !$this->sessionManager->has("NRUSUARIOAPP")) {
            throw new \Exception("Access denied.");
        }
    }
}
